@@ -27,10 +27,8 @@ autowrite(buf_T *buf, int forceit)
     bufref_T	bufref;
 
     if (!(p_aw || p_awa) || !p_write
-#ifdef FEAT_QUICKFIX
 	    // never autowrite a "nofile" or "nowrite" buffer
 	    || bt_dontwrite(buf)
-#endif
 	    || (!forceit && buf->b_p_ro) || buf->b_ffname == NULL)
 	return FAIL;
     set_bufref(&bufref, buf);
@@ -363,11 +361,10 @@ check_changed_any(
 	if (
 #ifdef FEAT_TERMINAL
 		term_job_running(buf->b_term)
-		    ? semsg(_("E947: Job still running in buffer \"%s\""),
-								  buf->b_fname)
+		    ? semsg(_(e_job_still_running_in_buffer_str), buf->b_fname)
 		    :
 #endif
-		semsg(_("E162: No write since last change for buffer \"%s\""),
+		semsg(_(e_no_write_since_last_change_for_buffer_str),
 		    buf_spname(buf) != NULL ? buf_spname(buf) : buf->b_fname))
 	{
 	    save = no_wait_return;
@@ -564,9 +561,11 @@ ex_listdo(exarg_T *eap)
 		    // Clear 'shm' to avoid that the file message overwrites
 		    // any output from the command.
 		    p_shm_save = vim_strsave(p_shm);
-		    set_option_value((char_u *)"shm", 0L, (char_u *)"", 0);
+		    set_option_value_give_err((char_u *)"shm",
+							  0L, (char_u *)"", 0);
 		    do_argfile(eap, i);
-		    set_option_value((char_u *)"shm", 0L, p_shm_save, 0);
+		    set_option_value_give_err((char_u *)"shm",
+							    0L, p_shm_save, 0);
 		    vim_free(p_shm_save);
 		}
 		if (curwin->w_arg_idx != i)
@@ -624,9 +623,9 @@ ex_listdo(exarg_T *eap)
 		// Go to the next buffer.  Clear 'shm' to avoid that the file
 		// message overwrites any output from the command.
 		p_shm_save = vim_strsave(p_shm);
-		set_option_value((char_u *)"shm", 0L, (char_u *)"", 0);
+		set_option_value_give_err((char_u *)"shm", 0L, (char_u *)"", 0);
 		goto_buffer(eap, DOBUF_FIRST, FORWARD, next_fnum);
-		set_option_value((char_u *)"shm", 0L, p_shm_save, 0);
+		set_option_value_give_err((char_u *)"shm", 0L, p_shm_save, 0);
 		vim_free(p_shm_save);
 
 		// If autocommands took us elsewhere, quit here.
@@ -646,9 +645,9 @@ ex_listdo(exarg_T *eap)
 		// Clear 'shm' to avoid that the file message overwrites
 		// any output from the command.
 		p_shm_save = vim_strsave(p_shm);
-		set_option_value((char_u *)"shm", 0L, (char_u *)"", 0);
+		set_option_value_give_err((char_u *)"shm", 0L, (char_u *)"", 0);
 		ex_cnext(eap);
-		set_option_value((char_u *)"shm", 0L, p_shm_save, 0);
+		set_option_value_give_err((char_u *)"shm", 0L, p_shm_save, 0);
 		vim_free(p_shm_save);
 
 		// If jumping to the next quickfix entry fails, quit here
@@ -754,14 +753,14 @@ ex_compiler(exarg_T *eap)
 		if (old_cur_comp != NULL)
 		    old_cur_comp = vim_strsave(old_cur_comp);
 		do_cmdline_cmd((char_u *)
-			      "command -nargs=* CompilerSet setlocal <args>");
+		   "command -nargs=* -keepscript CompilerSet setlocal <args>");
 	    }
 	    do_unlet((char_u *)"g:current_compiler", TRUE);
 	    do_unlet((char_u *)"b:current_compiler", TRUE);
 
 	    sprintf((char *)buf, "compiler/%s.vim", eap->arg);
 	    if (source_runtime(buf, DIP_ALL) == FAIL)
-		semsg(_("E666: compiler not supported: %s"), eap->arg);
+		semsg(_(e_compiler_not_supported_str), eap->arg);
 	    vim_free(buf);
 
 	    do_cmdline_cmd((char_u *)":delcommand CompilerSet");
